@@ -1,6 +1,12 @@
 from pycid.analyze.effects import introduced_total_effect, total_effect  # noqa
 from pycid.analyze.instrumental_control_incentive import admits_ici, admits_ici_list  # noqa
-from pycid.analyze.reasoning_patterns import manipulation, revealing_or_denying, signaling  # noqa
+from pycid.analyze.reasoning_patterns import (  # noqa
+    direct_effect,
+    get_reasoning_patterns,
+    manipulation,
+    revealing_or_denying,
+    signaling,
+)
 from pycid.analyze.requisite_graph import requisite, requisite_graph, requisite_list  # noqa
 from pycid.analyze.response_incentive import admits_ri, admits_ri_list  # noqa
 from pycid.analyze.value_of_control import (  # noqa
@@ -10,5 +16,6 @@ from pycid.analyze.value_of_control import (  # noqa
     admits_indir_voc_list,
     admits_voc,
     admits_voc_list,
+    quantitative_voc,
 )
-from pycid.analyze.value_of_information import admits_voi, admits_voi_list  # noqa
+from pycid.analyze.value_of_information import admits_voi, admits_voi_list, quantitative_voi  # noqa
