@@ -1,6 +1,14 @@
 from pycid.core.causal_bayesian_network import CausalBayesianNetwork  # noqa
 from pycid.core.cid import CID  # noqa
-from pycid.core.cpd import DecisionDomain, FunctionCPD, StochasticFunctionCPD, UniformRandomCPD  # noqa
+from pycid.core.cpd import (  # noqa
+    ConstantCPD,
+    DecisionDomain,
+    StochasticFunctionCPD,
+    bernoulli,
+    discrete_uniform,
+    function_copy,
+    noisy_copy,
+)
 from pycid.core.macid import MACID  # noqa
 from pycid.core.macid_base import MACIDBase, MechanismGraph  # noqa
 from pycid.core.relevance_graph import CondensedRelevanceGraph, RelevanceGraph  # noqa
