@@ -1,6 +1,7 @@
 from __future__ import annotations
 
 import itertools
+import math
 from functools import lru_cache
 from typing import (
     Callable,
@@ -8,14 +9,15 @@ from typing import (
     Dict,
     Hashable,
     Iterable,
+    Iterator,
     KeysView,
     List,
     Mapping,
     Optional,
+    Sequence,
     Set,
     Tuple,
     Union,
-    Iterator,
 )
 
 import matplotlib.cm as cm
@@ -24,8 +26,8 @@ import numpy as np
 from pgmpy.factors.discrete import TabularCPD
 from pgmpy.inference.ExactInference import BeliefPropagation
 
-from pycid.core.causal_bayesian_network import CausalBayesianNetwork
-from pycid.core.cpd import DecisionDomain, FunctionCPD, Outcome, UniformRandomCPD
+from pycid.core.causal_bayesian_network import CausalBayesianNetwork, Relationship
+from pycid.core.cpd import DecisionDomain, Outcome, StochasticFunctionCPD
 from pycid.core.relevance_graph import RelevanceGraph
 
 AgentLabel = Hashable  # Could be a TypeVar instead but that might be overkill
@@ -49,6 +51,20 @@ class MACIDBase(CausalBayesianNetwork):
         A dictionary mapping utility node label => agent label.
     """
 
+    class Model(CausalBayesianNetwork.Model):
+        def __setitem__(
+            self, variable: str, relationship: Union[Relationship, Sequence], sync_state_names: bool = False
+        ) -> None:
+            if isinstance(relationship, (DecisionDomain, Sequence)) and variable not in self.cbn.decisions:
+                raise ValueError(f"trying to add DecisionDomain to non-decision node {variable}")
+            super().__setitem__(variable, relationship, sync_state_names)
+
+        def to_tabular_cpd(self, variable: str, relationship: Union[Relationship, Sequence[Outcome]]) -> TabularCPD:
+            if isinstance(relationship, Sequence):
+                return DecisionDomain(variable, self.cbn, relationship)
+            else:
+                return super().to_tabular_cpd(variable, relationship)
+
     def __init__(
         self,
         edges: Iterable[Tuple[str, str]],
@@ -91,16 +107,18 @@ class MACIDBase(CausalBayesianNetwork):
         return self.agent_utilities.keys()
 
     def make_decision(self, node: str, agent: AgentLabel = 0) -> None:
-        """"Turn a chance or utility node into a decision node."""
+        """ "Turn a chance or utility node into a decision node.
+        - agent specifies which agent the decision node should belong to in a MACID.
+        """
         self.make_chance(node)
         self.agent_decisions[agent].append(node)
         self.decision_agent[node] = agent
         cpd = self.get_cpds(node)
         if cpd and not isinstance(cpd, DecisionDomain):
-            self.add_cpds(DecisionDomain(node, cpd.state_names[node]))
+            self.add_cpds(DecisionDomain(node, self, cpd.state_names[node]))
 
     def make_utility(self, node: str, agent: AgentLabel = 0) -> None:
-        """"Turn a chance or utility node into a decision node."""
+        """ "Turn a chance or utility node into a decision node."""
         self.make_chance(node)
         self.agent_utilities[agent].append(node)
         self.utility_agent[node] = agent
@@ -116,18 +134,8 @@ class MACIDBase(CausalBayesianNetwork):
             agent = self.utility_agent.pop(node)
             self.agent_utilities[agent].remove(node)
 
-    def add_cpds(self, *cpds: TabularCPD) -> None:
-        """
-        Add the given CPDs and initialize them.
-
-        A CPD can be a TabularCPD, FunctionCPD, UniformRandomCPD, DecisionDomain, or a StochasticFunctionCPD.
-        """
-
-        # For MACIDs and CIDs we need to do an extra check
-        for cpd in cpds:
-            if isinstance(cpd, DecisionDomain) and cpd.variable not in self.decisions:
-                raise ValueError(f"trying to add DecisionDomain to non-decision node {cpd.variable}")
-        super().add_cpds(*cpds)
+    def add_cpds(self, *cpds: TabularCPD, **relationships: Union[Relationship, List[Outcome]]) -> None:
+        super().add_cpds(*cpds, **relationships)
 
     def query(
         self, query: Iterable[str], context: Dict[str, Outcome], intervention: Dict[str, Outcome] = None
@@ -146,12 +154,12 @@ class MACIDBase(CausalBayesianNetwork):
         intervention: Interventions to apply. A dictionary mapping node => outcome.
         """
 
+        self._fix_lowercase_variables(context)
         for variable, outcome in context.items():
-            if outcome not in self.get_cpds(variable).state_names[variable]:
+            if outcome not in self.get_cpds(variable).domain:
                 raise ValueError(f"The outcome {outcome} is not in the domain of {variable}")
 
-        if intervention is None:
-            intervention = {}
+        intervention = intervention or {}
 
         # Check that strategically relevant decisions have a policy specified
         mech_graph = MechanismGraph(self)
@@ -246,9 +254,9 @@ class MACIDBase(CausalBayesianNetwork):
 
     def sufficient_recall(self, agent: Optional[AgentLabel] = None) -> bool:
         """
-        Finds whether a (MA)CID has sufficient recall.
+        Returns true if the agent has sufficient recall in the (MA)CID.
 
-        Agent i in the MAID has sufficient recall if the relevance graph
+        Agent i in the (MA)CID has sufficient recall if the relevance graph
         restricted to contain only i's decision nodes is acyclic.
 
         If an agent is specified, sufficient recall is checked only for that agent.
@@ -267,10 +275,10 @@ class MACIDBase(CausalBayesianNetwork):
                 return False
         return True
 
-    def pure_decision_rules(self, decision: str) -> Iterator[FunctionCPD]:
+    def pure_decision_rules(self, decision: str) -> Iterator[StochasticFunctionCPD]:
         """Return a list of the decision rules available at the given decision"""
 
-        domain = self.get_cpds(decision).state_names[decision]
+        domain = self.model.domain[decision]
         parents = self.get_parents(decision)
         parent_cardinalities = [self.get_cardinality(parent) for parent in parents]
 
@@ -294,20 +302,23 @@ class MACIDBase(CausalBayesianNetwork):
                 # using a default argument is a trick to get func_list to evaluate early
                 return lambda **parent_values: early_eval_func_list[arg2idx(parent_values)]
 
-            yield FunctionCPD(decision, produce_function(), domain=domain)
+            yield StochasticFunctionCPD(decision, produce_function(), self, domain=domain)
 
-    def pure_policies(self, decision_nodes: Iterable[str]) -> Iterator[Tuple[FunctionCPD, ...]]:
+    def pure_policies(self, decision_nodes: Iterable[str]) -> Iterator[Tuple[StochasticFunctionCPD, ...]]:
         """
         Iterate over all of an agent's pure policies in this subgame.
         """
         possible_dec_rules = list(map(self.pure_decision_rules, decision_nodes))
         return itertools.product(*possible_dec_rules)
 
-    def optimal_pure_policies(self, decisions: Iterable[str], eps: float = 1e-8) -> List[Tuple[FunctionCPD, ...]]:
+    def optimal_pure_policies(
+        self, decisions: Iterable[str], rel_tol: float = 1e-9
+    ) -> List[Tuple[StochasticFunctionCPD, ...]]:
         """Find all optimal policies for a given set of decisions.
 
         - All decisions must belong to the same agent.
-        - eps is the margin of error when comparing utilities for equality.
+        - rel_tol: is the relative tolerance. It is the amount of error allowed, relative to the larger
+        absolute value of the two values it is comparing (the two utilities.)
         """
         if not decisions:
             return []
@@ -331,27 +342,29 @@ class MACIDBase(CausalBayesianNetwork):
         for policy in macid.pure_policies(decisions):
             macid.add_cpds(*policy)
             expected_utility = macid.expected_utility({}, agent=agent)
-            if abs(expected_utility - max_utility) < eps:
+            if math.isclose(expected_utility, max_utility, rel_tol=rel_tol):
                 optimal_policies.append(policy)
             elif expected_utility > max_utility:
                 optimal_policies = [policy]
                 max_utility = expected_utility
         return optimal_policies
 
-    def optimal_pure_decision_rules(self, decision: str) -> List[FunctionCPD]:
+    def optimal_pure_decision_rules(self, decision: str) -> List[StochasticFunctionCPD]:
         """
-        Return a list of all optimal decision rules for a given decision
+        Return a list of all optimal pure decision rules for a given decision
         """
         return [policy[0] for policy in self.optimal_pure_policies([decision])]
 
     def impute_random_decision(self, d: str) -> None:
         """Impute a random policy to the given decision node"""
-        current_cpd = self.get_cpds(d)
-        if current_cpd:
-            sn = current_cpd.state_names[d]
-        else:
+        try:
+            domain = self.model.domain[d]
+        except KeyError:
             raise ValueError(f"can't figure out domain for {d}, did you forget to specify DecisionDomain?")
-        self.add_cpds(UniformRandomCPD(d, sn))
+        else:
+            self.model[d] = StochasticFunctionCPD(
+                d, lambda **pv: {outcome: 1 / len(domain) for outcome in domain}, self, domain, label="random_decision"
+            )
 
     def impute_fully_mixed_policy_profile(self) -> None:
         """Impute a fully mixed policy profile - ie a random decision rule to all decision nodes"""
@@ -359,7 +372,7 @@ class MACIDBase(CausalBayesianNetwork):
             self.impute_random_decision(d)
 
     def remove_all_decision_rules(self) -> None:
-        """ Remove the decision rules from all decisions in the (MA)CID"""
+        """Remove the decision rules from all decisions in the (MA)CID"""
         for d in self.decisions:
             cpd = self.get_cpds(d)
             if cpd is None:
@@ -367,46 +380,40 @@ class MACIDBase(CausalBayesianNetwork):
             elif isinstance(cpd, DecisionDomain):
                 continue
             else:
-                self.add_cpds(DecisionDomain(d, cpd.domain))
+                self.add_cpds(DecisionDomain(d, self, cpd.domain))
 
-    def impute_optimal_decision(self, d: str) -> None:
+    def impute_optimal_decision(self, decision: str) -> None:
         """Impute an optimal policy to the given decision node"""
         # self.add_cpds(random.choice(self.optimal_pure_decision_rules(d)))
-        self.impute_random_decision(d)
-        cpd = self.get_cpds(d)
-        parents = cpd.variables[1:]
-        idx2name = cpd.no_to_name[d]
-        utility_nodes = self.agent_utilities[self.decision_agent[d]]
-        descendant_utility_nodes = list(set(utility_nodes).intersection(nx.descendants(self, d)))
-        new = self.copy()  # using a copy "freezes" the policy so it doesn't adapt to future interventions
+        self.impute_random_decision(decision)
+        domain = self.model.domain[decision]
+        utility_nodes = self.agent_utilities[self.decision_agent[decision]]
+        descendant_utility_nodes = list(set(utility_nodes).intersection(nx.descendants(self, decision)))
+        copy = self.copy()  # using a copy "freezes" the policy so it doesn't adapt to future interventions
 
         @lru_cache(maxsize=1000)
         def opt_policy(**parent_values: Outcome) -> Outcome:
-            nonlocal descendant_utility_nodes
-            context: Dict[str, Outcome] = {p: parent_values[p.lower()] for p in parents}
-            eu = []
-            for d_idx in range(new.get_cardinality(d)):
-                context[d] = idx2name[d_idx]
-                eu.append(sum(new.expected_value(descendant_utility_nodes, context)))
-            return idx2name[np.argmax(eu)]
+            eu = {}
+            for d in domain:
+                parent_values[decision] = d
+                eu[d] = sum(copy.expected_value(descendant_utility_nodes, parent_values))
+            return max(eu, key=eu.get)  # type: ignore
 
-        self.add_cpds(FunctionCPD(d, opt_policy, domain=cpd.state_names[d], label="opt"))
+        self.add_cpds(StochasticFunctionCPD(decision, opt_policy, self, domain=domain, label="opt"))
 
-    def impute_conditional_expectation_decision(self, d: str, y: str) -> None:
-        """Imputes a policy for d = the expectation of y conditioning on d's parents"""
+    def impute_conditional_expectation_decision(self, decision: str, y: str) -> None:
+        """Imputes a policy for decision = the expectation of y conditioning on d's parents"""
         # TODO: Move to analyze, as this is not really a core feature?
-        parents: List[str] = self.get_parents(d)
-        new = self.copy()
+        copy = self.copy()
 
         @lru_cache(maxsize=1000)
         def cond_exp_policy(**pv: Outcome) -> float:
             if y.lower() in pv:
                 return pv[y.lower()]  # type: ignore
             else:
-                context = {p: pv[p.lower()] for p in parents}
-                return new.expected_value([y], context)[0]
+                return copy.expected_value([y], pv)[0]
 
-        self.add_cpds(FunctionCPD(d, cond_exp_policy, label="cond_exp({})".format(y)))
+        self.add_cpds(StochasticFunctionCPD(decision, cond_exp_policy, self, label="cond_exp({})".format(y)))
 
     # Wrapper around DAG.active_trail_nodes to accept arbitrary iterables for observed.
     # Really, DAG.active_trail_nodes should accept Sets, especially since it does
